from math import cos, sqrt, pi


class CoordinateException(Exception):
    pass


class CoordinateStringMalformedException(CoordinateException):
    def __init__(self, malformed_string):
        self.malformed_string = malformed_string

    def __str__(self):
        return "Error while attempting to convert string '{}' to Coordinate object".format(
            self.malformed_string
        )


class Coordinate:
    def __init__(self, latitude, longitude, altitude):
        self.latitude = latitude
        self.longitude = longitude
        self.altitude = altitude

    @staticmethod
    def from_string(coordinate_string):
        # coordinate_string should be formatted as 'xx.xxxxx,yy.yyyyyy,z.zzzzzz'
        try:
            lat_str, long_str, alt_str = coordinate_string.split(",")
        except ValueError:
            raise CoordinateStringMalformedException(coordinate_string)

        def to_float(val_string):
            return float(val_string.strip())

        return Coordinate(to_float(lat_str), to_float(long_str), to_float(alt_str))

    def to_string(self):
        def to_string(val_float):
            return "{:.8}".format(val_float)

        return ",".join(
            [to_string(val) for val in [self.latitude, self.longitude, self.altitude]]
        )

    def __sub__(self, other):
        """ Return the distance in meter between self and other
        """
        R = 6400e3

        def degree_to_radian(deg):
            return deg * pi / 180

        DY = degree_to_radian(self.latitude - other.latitude) * R
        DX = (
            degree_to_radian(self.longitude - other.longitude)
            * R
            * cos(degree_to_radian(other.latitude))
        )

        return sqrt((DY ** 2) + (DX ** 2))

    def __str__(self):
        return "Coordinate<lat:{}, long:{}, alt:{}>".format(
            self.latitude, self.longitude, self.altitude
        )

    @staticmethod
    def from_nmea_format(latitude_nmea, longitude_nmea, altitude_nmea):
        latitude_decimal_degree = Coordinate.nmea_format_to_deciman_degree(
            latitude_nmea
        )
        longitude_decimal_degree = Coordinate.nmea_format_to_deciman_degree(
            longitude_nmea
        )
        return Coordinate(
            latitude=latitude_decimal_degree,
            longitude=longitude_decimal_degree,
            altitude=altitude_nmea,
        )

    @staticmethod
    def nmea_format_to_deciman_degree(nmea_format):
        degree = int(nmea_format / 100)
        minute = nmea_format - (100 * degree)
        return Coordinate.degree_minutes_to_decimal_degree(degree=degree, minute=minute)

    @staticmethod
    def degree_minutes_to_decimal_degree(degree, minute):
        """ Convert a degree, minute, fraction of minute format into decimal degree
        """
        return degree + (minute / 60)
