from pynmea2 import parse, ChecksumError
from pynmea2.nmea import ParseError


class NmeaException(Exception):
    pass


class NmeaWrongChecksumException(NmeaException):
    def __init__(self, raw_nmea):
        self._raw_nmea = raw_nmea
        super(NmeaWrongChecksumException, self).__init__()

    def __str__(self):
        return "Checksum is wrong or missing: '{}'".format(self._raw_nmea)


class NmeaFrameUnknownSentenceType(NmeaException):
    def __init__(self, unknown_sentence_type):
        self.unknown_sentence_type = unknown_sentence_type
        super(NmeaFrameUnknownSentenceType, self).__init__()

    def __str__(self):
        return "Unknown NMEA sentence type '{}' (known types: {})".format(
            self.unknown_sentence_type, NmeaFrame.KNOWN_SENTENCE_TYPES
        )


class NmeaParseError(NmeaException):
    def __init__(self, raw_nmea):
        self.raw_nmea = raw_nmea

    def __str__(self):
        return "Failed to parse NMEA sentence '{}'".format(self.raw_nmea)


class NmeaFrame:
    KNOWN_SENTENCE_TYPES = ["GGA"]

    def __init__(self, raw_str):
        self._raw_str = raw_str
        self._parsed_nmea = None

    def __str__(self):
        return self._raw_str.strip()

    def parse(self):
        try:
            self._parsed_nmea = parse(self._raw_str, check=True)
        except ChecksumError:
            raise NmeaWrongChecksumException(self._raw_str)
        except ParseError:
            raise NmeaParseError(self._raw_str)
        sentence_type = self._parsed_nmea.sentence_type
        NmeaFrame.check_known_sentence_type_or_raise_error(sentence_type)

    @staticmethod
    def check_known_sentence_type_or_raise_error(sentence_type):
        if sentence_type not in NmeaFrame.KNOWN_SENTENCE_TYPES:
            raise NmeaFrameUnknownSentenceType(sentence_type)

    @property
    def longitude(self):
        if self._parsed_nmea.lon_dir == "W":
            return -1 * float(self._parsed_nmea.lon)
        else:
            return float(self._parsed_nmea.lon)

    @property
    def latitude(self):
        return float(self._parsed_nmea.lat)

    @property
    def altitude(self):
        return float(self._parsed_nmea.altitude)

    @property
    def timestamp(self):
        return self._parsed_nmea.timestamp

    @property
    def raw_str(self):
        return self._raw_str

    @raw_str.setter
    def raw_str(self, value):
        self._raw_str = value
        self._parsed_nmea = None
