class WifiChannel:
    """ Wifi Channel

    Represents a wifi channel with its name and frequency

    Attributes:
        name (str): The name of the channel
        frequency_mhz (int): The center frequency of the channel [MHz]

    """

    def __init__(self, channel_name, channel_frequency_mhz):
        self.name = channel_name
        self.frequency_mhz = channel_frequency_mhz

    def __str__(self):
        return "{}".format(self.name)

    def __repr__(self):
        return "{}: {}".format(self.name, self.frequency_mhz)


class WifiChannelsException(Exception):
    """ Base exception for WifiChannel class

    """

    pass


class WifiChannelsNameUnknownException(WifiChannelsException):
    """Exception describing an attempt to get a channel which name is unknown

    Attributes:
        channel_name (str): The unknown channel name

    """

    def __init__(self, channel_name):
        self.channel_name = channel_name

    def __str__(self):
        return "Channel name unknown: '{}'".format(self.channel_name)


class WifiChannels:
    """ Lists all wifi channels

    Attributes:
        WIFI_CHANNELS ([WifiChannel]): The list of all Wifi channels
    """

    WIFI_CHANNELS = [
        WifiChannel("CHANNEL_{}".format(index), freq)
        for index, freq in enumerate(list(range(2412, 2473, 5)) + [2484], 1)
    ]

    @staticmethod
    def get_channel_from_name(name):
        """ Search a channel object from its name

        Raises a WifiChannelsNameUnknownException exception if name
        cannot be found.

        Args:
            name (str): The name of the channel to search

        Returns:
            WifiChannel: The Wifi channel object with the corresponding name

        """
        try:
            return [chan for chan in WifiChannels.WIFI_CHANNELS if chan.name == name][0]
        except IndexError:
            raise WifiChannelsNameUnknownException(name)

    @staticmethod
    def get_channel_from_frequency(frequency_mhz):
        """ Search a channel object from its frequency

        TODO: Raise exception if frequency not found

        Args:
            frequency_mhz (int): The frequency of the channel to search [MHz]

        Returns:
            WifiChannel: The Wifi channel object with the corresponding name

        """
        return [
            chan
            for chan in WifiChannels.WIFI_CHANNELS
            if chan.frequency_mhz == frequency_mhz
        ][0]
