import re
from datetime import datetime
from lr1110evk.BaseTypes import (
    ScannedGnss,
    ScannedMacAddress,
    ScannedMacAddressMalformedCsvLineException,
    Version,
    ScannedGnssMalformedCsvLineException,
)
from collections import namedtuple
from itertools import groupby
from lr1110evk.Job import ResultLogger
from lr1110evk.FieldTestPost.Core.ExternalCoordinate import (
    ExternalCoordinate,
    ExternalCoordinateMalformedCsvLineException,
)


ResultLine = namedtuple("ResultLine", ["date", "job_counter", "job_id", "scan_info"])


class FileReader:
    DATE_FORMAT = "%Y-%m-%d %H:%M:%S.%f"
    VERSION_TOKEN = "# version:"
    REGEXP_LOG = re.compile(
        "^\[(?P<date_info>[^\]]+)\] \[(?P<job_counter>[0-9]+) - (?P<job_id>[0-9]+)\] ?(?P<scan_info>.*)$"
    )

    def __init__(self):
        pass

    @staticmethod
    def parse_file(filename):
        scanned_results = list()
        version = None
        with open(filename, "r") as f:
            lines = f.readlines()
        for line in lines:
            if line.startswith(FileReader.VERSION_TOKEN):
                version = Version.from_metadata(line[len(FileReader.VERSION_TOKEN) :])
            else:
                try:
                    result_line = FileReader.get_scan_info_and_job_data_from_line(line)
                except Exception as malformed_except:
                    print(
                        "Failed to parse line: {} ('{}')".format(line, malformed_except)
                    )
                    result_line = None
                if result_line:
                    scanned_results.append(result_line)
        return scanned_results, version

    @staticmethod
    def get_scan_info_and_job_data_from_line(str_line):
        match_line = FileReader.REGEXP_LOG.match(str_line)
        date_scan_str = match_line.group("date_info")
        job_counter = int(match_line.group("job_counter"))
        job_id = int(match_line.group("job_id"))
        scan_line = match_line.group("scan_info")
        date_scan = datetime.strptime(date_scan_str, FileReader.DATE_FORMAT)
        if (
            scan_line == ResultLogger.NO_RESULT_TOKEN
            or scan_line == ResultLogger.EXCEPTION_TOKEN
        ):
            scan = f"Scan exception: {scan_line}"
        else:
            try:
                scan = ScannedMacAddress.from_csv_line(date_scan, scan_line)
            except ScannedMacAddressMalformedCsvLineException:
                try:
                    scan = ScannedGnss.from_csv_line(date_scan, scan_line)
                except ScannedGnssMalformedCsvLineException:
                    try:
                        scan = ExternalCoordinate.from_csv_line(scan_line)
                    except ExternalCoordinateMalformedCsvLineException as malformed_exc:
                        scan = f"Malformed result: {malformed_exc.malformed_csv_line}"
        result_line = ResultLine(
            date=date_scan, job_counter=job_counter, job_id=job_id, scan_info=scan
        )
        return result_line

    @staticmethod
    def generate_result_groups(result_lines):
        def group_key(res):
            group_val = res.job_counter
            return group_val

        sorted_result_lines = sorted(result_lines, key=group_key)
        key_scan_result_groups = groupby(sorted_result_lines, group_key)
        return key_scan_result_groups
