from yaml import load_all
from pkg_resources import resource_string
from unidecode import unidecode


class GeocodingBuilder:
    HOST_ASSET_DIR = "assets"
    GEOCODING_COMPONENT_FILENAME = "geocoding_components.yaml"

    ROAD_KEY_NAME = "road"
    CITY_KEY_NAME = "city"
    COUNTRY_KEY_NAME = "country"
    COUNTRY_CODE_KEY_NAME = "country_code"
    STATE_KEY_NAME = "state"
    POSTCODE_KEY_NAME = "postcode"
    HOUSE_KEY_NAME = "house"

    def __init__(self):
        self.geo_coding_file = GeocodingBuilder.get_resource_path()
        self.name_aliases = None

    def load_names_aliases(self):
        self.name_aliases = [doc for doc in load_all(self.geo_coding_file)]

    def build_data(self, osm_json):
        road = GeocodingBuilder.extract_from_name_or_none(
            self.name_aliases, GeocodingBuilder.ROAD_KEY_NAME, osm_json
        )
        city = GeocodingBuilder.extract_from_name_or_none(
            self.name_aliases, GeocodingBuilder.CITY_KEY_NAME, osm_json
        )
        country = GeocodingBuilder.extract_from_name_or_none(
            self.name_aliases, GeocodingBuilder.COUNTRY_KEY_NAME, osm_json
        )
        country_code = GeocodingBuilder.extract_from_name_or_none(
            self.name_aliases, GeocodingBuilder.COUNTRY_CODE_KEY_NAME, osm_json
        )
        state = GeocodingBuilder.extract_from_name_or_none(
            self.name_aliases, GeocodingBuilder.STATE_KEY_NAME, osm_json
        )
        postcode = GeocodingBuilder.extract_from_name_or_none(
            self.name_aliases, GeocodingBuilder.POSTCODE_KEY_NAME, osm_json
        )
        house = GeocodingBuilder.extract_from_name_or_none(
            self.name_aliases, GeocodingBuilder.HOUSE_KEY_NAME, osm_json
        )
        data = GeoCodingData(road, city, country, country_code, state, postcode, house)
        return data

    @staticmethod
    def extract_from_name_or_none(name_aliases, name, osm_dict):
        if name_aliases:
            try:
                aliases = [al["aliases"] for al in name_aliases if al["name"] == name][
                    0
                ]
            except KeyError:
                aliases = list()
            keys = [name] + aliases
        else:
            keys = [name]
        val = "dnf"
        for key in keys:
            try:
                val = osm_dict[key]
            except KeyError:
                pass
            else:
                break
        return val

    @staticmethod
    def get_resource_path():
        resource_package = "lr1110evk"
        resource_geo_coding_file_package_path = "/".join(
            (
                GeocodingBuilder.HOST_ASSET_DIR,
                GeocodingBuilder.GEOCODING_COMPONENT_FILENAME,
            )
        )
        file_path = resource_string(
            resource_package, resource_geo_coding_file_package_path
        )
        return file_path


class GeoCodingData:
    def __init__(self, street, town, country, country_code, state, postcode, house):
        self._street = street
        self._town = unidecode(town)
        self._country = unidecode(country)
        self._country_code = country_code.upper()
        self._state = state
        self._postcode = postcode
        self._house = house

    def __str__(self):
        return "{item._street},{item._town},{item._country_code}".format(item=self)

    @property
    def street(self):
        return self._street

    @property
    def town(self):
        return self._town

    @property
    def country(self):
        return self._country

    @property
    def country_code(self):
        return self._country_code

    @property
    def state(self):
        return self._state

    @property
    def postcode(self):
        return self._postcode

    @property
    def house(self):
        return self._house
