from .JobReader import JobReader
from ..SerialExchange import (
    SerialHandler,
    CommunicationHandler,
    CommunicationHandlerException,
    CommunicationHandlerSerialNotListeningException,
)
from .JobExecutor import JobExecutor, JobExecutorException, JobResetFailed
from .JobJsonValidator import JobValidator


class ExecutorException(Exception):
    pass


class ExecutorCriticalException(ExecutorException):
    def __init__(self, exception):
        self.exception = exception

    def __str__(self):
        return "Critical job executor exception: {}".format(self.exception)


class Executor:
    def __init__(self, result_logger, debug_logger):
        self.job_reader = JobReader()
        self.serial_handler = SerialHandler()
        self.communication_handler = CommunicationHandler(
            self.serial_handler, debug_logger
        )
        self.job_executor = JobExecutor(self.communication_handler, debug_logger)
        self.result_logger = result_logger
        self.debug_logger = debug_logger
        self.job_execution_counter = 0

    def stop(self):
        self.communication_handler.stop()

    def load_jobs_from_file(self, job_filename):
        JobValidator.validate_job_json_file(job_filename)
        self.job_reader.read_from_file(job_filename)

    def load_jobs_from_job_list(self, job_list, infinite_execution):
        self.job_reader.jobs.extend(job_list)
        self.job_reader.infinite_execution = infinite_execution

    def connect_serial(self):
        port = SerialHandler.discover(SerialHandler.DISCOVER_PORT_REGEXP)
        device = port.device
        self.serial_handler.set_serial_port(device)

    def execute(self):
        self.communication_handler.start()
        self.communication_handler.wait_embedded_to_be_configured_for_field_test()
        self.get_and_save_version_info()
        if self.job_reader.infinite_execution:
            while True:
                self.execute_jobs()
        else:
            self.execute_jobs()
        self.stop()

    def reset(self):
        self.log("Attempting reset")
        try:
            self.job_executor.reset()
        except JobResetFailed:
            self.exception("Reset target while stop failed")

    def hard_reset(self):
        # TODO: implement hard reset
        self.log("Unimplemented Hard Reset")

    def get_and_save_version_info(self):
        version = self.job_executor.get_version_info()
        self.result_logger.log_metadata("version: {}".format(version.to_metadata()))

    def execute_jobs(self):
        for job in self.job_reader.jobs:
            self.job_execution_counter += 1
            self.log(
                "Execute job #{}: [{}]".format(self.job_execution_counter, str(job))
            )
            execute_post_reset = False
            try:
                results = self.job_executor.execute_job(job)
            except JobExecutorException as job_exc:
                # That's a high level exception: link is ok, but command is
                # unknown
                self.result_logger.log_exception(job.id, self.job_execution_counter)
                self.exception("Job failed with error: {}".format(str(job_exc)))
                execute_post_reset = True
            except CommunicationHandlerSerialNotListeningException as com_not_listening:
                self.result_logger.log_exception(job.id, self.job_execution_counter)
                execute_post_reset = False  # We don't try a post reset here as the exception comes from the fact the thread managing the serial is dead
                self.exception(
                    "Communication failure: {}".format(str(com_not_listening))
                )
                raise ExecutorCriticalException(com_not_listening)
            except CommunicationHandlerException as com_hand_exc:
                # That's a low level exception: link is not ok
                self.result_logger.log_exception(job.id, self.job_execution_counter)
                execute_post_reset = True
                self.exception("Communication failure: {}".format(str(com_hand_exc)))
            else:
                # Save results
                if results:
                    self.log(
                        "[RESULTS] Job {}({}): {} result(s)\n  - {}".format(
                            job.id,
                            job.name,
                            len(results),
                            "\n  - ".join(["{}".format(res) for res in results]),
                        )
                    )
                self.save_results(results, job.id, self.job_execution_counter)
            if execute_post_reset:
                try:
                    self.communication_handler.empty_fifo()
                    self.reset()
                except JobResetFailed:
                    self.exception("Soft reset failed, attempting Hard reset...")
                    self.hard_reset()
            self.log("Terminate job: {}".format(str(job)))

    def save_results(self, results, job_id, job_counter):
        if not results:
            self.result_logger.log_no_result(job_id, job_counter)
        for result in results:
            try:
                self.result_logger.log_wifi(result, job_id, job_counter)
            except AttributeError:
                self.result_logger.log_gnss(result, job_id, job_counter)

    def log(self, msg):
        self.debug_logger.log("[LOG] {}".format(msg))

    def exception(self, msg):
        self.debug_logger.log("[EXCEPTION] {}".format(msg))
